!****   himpose.f90
! compiled with pgf90 -byteswapio (for SGI byte structured files on Linux PCs)

      program himpose
      implicit none
      
      character*60 filein, symin, mapout, sigmaout
      character*80 arg
      character*60 comment
      character ichar
      character*83 string
      logical existence
      integer npar, j, nzlim, nx, ny, irmin, irmax, dum1, dum2
      integer nsec, nheadbytes, n_aniso, nheadrec, nphi, nreclength
      real rmin, rmax, scale, pi, twopi, zstart, deltaz, deltaphi
      real header(25), header2(4096), head(10000)
      real, allocatable :: image1(:), image2(:), dencyl(:)
      real, allocatable :: imageout(:), symcyl(:)

!***    input parameters:
!           filein (SPIDER image format density)
!           symin  (SPIDER document format for symmetry)
!           mapout (SPIDER image format density)
!           scale  (sampling in data, A/pixel)
!           rmin   (minimum radius to evaluate in helix, in Angtsroms)
!           rmax   (maximum radius to evaluate in helix, in Angtsroms)


      npar = iargc()
      
      do j = 1, npar
         call getarg(j, arg)
         if (j == 1) then
            read (arg,*) filein
         else if (j == 2) then
            read (arg,*) symin
         else if (j == 3) then
            read (arg,*) mapout
         else if (j == 4) then
            read (arg,*) scale
         else if (j == 5) then
            read (arg,*) rmin
         else if (j == 6) then
            read (arg,*) rmax
         end if
      enddo
      
      pi = 3.14159
      twopi = 2.*pi
      
!***  read phistart, zstart from doc file
      
      open(11, file=symin, status='old')
      
!***  read comment line
      read(11,1111)comment
 1111 format(a)
      
!***  move to end of file
 1    read(11,*,end=99) dum1, dum2, deltaphi, zstart
      go to 1
 99   continue
      
      close(11)
      
      open(1, file=filein, form='unformatted', access='direct',
     $     recl=128, status='old')
      
      read(1,REC=1)header

      nsec=header(1)
      ny=header(2)
      nx=header(12)
      nheadrec=header(13)
      nheadbytes=header(22)
      nreclength=header(23)

      close(1)

!     irmin = min radius (in pixels) in cylindrical coordinates
!     irmax = max radius (in pixels) in cylindrical coordinates
!     nphi  = number of samples in phi in cylindrical coordinates
      irmin = nint(rmin/scale)
      irmax = nint(rmax/scale)
      
      nphi = twopi*rmax/scale

!     deltaz is rise per subunit (in pixels)
      deltaz=zstart/scale
      
!     nzlim will be the number of substeps taken
      nzlim=nint(deltaz + 2.0)
      
!     n_aniso will be dimension in z for anisotropic symmetrized volume
      n_aniso = 10 + nzlim*(1 + (ny/deltaz))
      
!     Dynamically allocate arrays with correct sizes
      allocate(image1(nx*ny*nsec))
      allocate(image2(nsec*nx*ny))
      allocate(dencyl((irmax-irmin+1)*nphi*ny))
      allocate(imageout(nx*nx*n_aniso))
      allocate(symcyl((irmax-irmin+1)*nphi*nzlim))

      call spider_read(nx, ny, nsec, nheadrec, nreclength, image1,
     $     header2, filein)

      call work(nx, ny, nsec, image1, image2, dencyl, 
     $     scale, irmin, irmax, nphi)
      
!***  have density in cylindrical coords., now impose symmetry
      call symmetry(dencyl, symcyl, nx, ny, scale, imageout, image1,
     $     deltaphi, zstart, irmin, irmax, nphi, nzlim, n_aniso)
      
!**   
!     write symmetrized image
      call spider_write(nx, nx, ny, image1, head, mapout)

      end program himpose      
      

!******************************************************
      subroutine spider_read(nxin, nyin, nsec, nheadrec, nreclength,
     $  image, header, filename)
      implicit none

      character*60 filename
      real image(nxin, nyin, nsec), header(nreclength/4)
      integer nxin, nyin, nsec, nheadrec, nreclength
      integer i, j, k, ii
      logical iset

      open(1, file=filename, form='unformatted', access='direct',
     $     recl=nreclength, status='old')
      
!***  read in header
      do i=1, nheadrec
         read(1, REC=i)header
      enddo
      
!***  read in image data
      ii = nheadrec
      do k=1,nsec
         do j=1,nyin
            ii = ii + 1
            read(1, REC=ii)(image(i,j,k), i=1,nxin)
         enddo
      enddo
      
      close(1)
      
      return
      end subroutine spider_read


!********************************************************
      subroutine work(nx, ny, nsec, image1, image2, dencyl, scale,
     $     irmin, irmax, nphi)
      implicit none

      integer nx, ny, nz, nsec, irmin, irmax, nphi
      real scale, a, b, del1, dely, deltax, r
      real*4 image1(nx,ny,nsec), image2(nsec,nx,ny)
      real*4 dencyl(irmin:irmax,nphi,ny)
      integer i, j, k, ir, ix, iy, iphi
      real pi, twopi, x, y, xcen, ycen, dgr, phiinc
      real phi, csphi(nphi), snphi(nphi)

      pi = 3.14159
      twopi = 2.*pi
      dgr=180./pi

      phiinc = twopi/nphi
      
!**** rotate by 90 degrees so helical axis will be along z
      do k=1,ny
         do j=1,nx
            do i=1,nsec
               image2(i,j,k) = image1(j,k,i)
            enddo
         enddo
      enddo
      
      xcen=(nsec/2) + 1
      ycen=(nx/2) + 1

!**   convert to cylindrical coords
      do k = 1,ny
         do iphi = 1,nphi
            do ir = irmin,irmax
               dencyl(ir,iphi,k) = 0.0
            enddo
         enddo
      enddo

      do iphi=1,nphi
         phi=(iphi-1)*phiinc
         csphi(iphi) = cos(phi)
         snphi(iphi) = sin(phi)
      enddo

      do k=1,ny
         do iphi=1,nphi
            
            do ir= irmin,irmax
               r = float(ir)
               
               x = xcen + r*csphi(iphi)
               y = ycen + r*snphi(iphi)
               
!**** use bilinear interpolation to find density in old Cartesian coordinates for this r,phi point
               ix=int(x)
               iy=int(y)
               if(ix < 1 .or. ix >= nsec)then
                  print *,'ix=',ix
                  cycle  ! fortran 90 - end iteration early
               end if
               
               if(iy < 1 .or .iy>= nx)then
                  print *,'iy=',iy
                  cycle  ! fortran 90 - end iteration early
               end if
               
               deltax = x-real(ix)
               dely   = y-real(iy)
               del1=1.-deltax
               a=(del1*image2(ix,iy,k)+deltax*image2(ix+1,iy,k))*
     $              (1.-dely)
               b=(del1*image2(ix,iy+1,k)+deltax*image2(ix+1,iy+1,k))*
     $              dely
               
               dencyl(ir,iphi,k)=a + b
               
            enddo
         enddo
      enddo

      end subroutine work

!*********************************************
      subroutine symmetry(dencyl, symcyl, nx, nz, scale, imageout,
     $     resampout, deltaphi, zstart, irmin, irmax, nphi, nzlim,
     $     n_aniso)
      implicit none

      integer nx, nz
      integer irmin, irmax, nphi, nzlim, n_aniso
      real z, zdif, zwant, zbase, zratio, xcen, ycen, y, y2, x, x2
      real pi, twopi, r, r2, radlim1, radlim2, phiinc
      real deltaphi, zstart, scale
      real dir, dsum, dsum2, diz, density, dgr, den1, den2
      real deltaz, deltadeltaz
      integer nzstep, nsubl, ny, nsubnew, i, j, k, krep, iphip
      integer nxstep, ir, iz, kz, iphi
      real*4 dencyl(irmin:irmax,nphi,nz), symcyl(irmin:irmax,nphi,nzlim)
      real*4 imageout(nx,nx,n_aniso), resampout(nx,nx,nz)

      pi = 3.14159
      twopi = 2.*pi
      dgr = twopi/360.
      
      phiinc = twopi/nphi

      deltaz=zstart/scale
      
!     *       deltaz is rise in pixels; we want deltadeltaz to be less than 1 pixel
!     *       nzlim will be the number of substeps taken
      
      deltadeltaz=real(deltaz/nzlim)
      
!***  nsubl is the number of subunits to average
      nsubl=nint( (2.*nz)/(3.*deltaz) )
      
!     *       zbase is starting position in z (in pixels), to exclude end effects
      
      zbase=(nz-nsubl*deltaz)/2.

!     *       initialize symmetrized volume array
      do nzstep=1,nzlim
         do iphi=1,nphi
            do ir=irmin,irmax
               symcyl(ir, iphi, nzstep) = 0.0
            enddo
         enddo
      enddo
      
!******************

      do nzstep=1,nzlim
         do iphi=1,nphi
            do ir=irmin,irmax
               
!***  loop over nsubl subunits
               dsum=0.0
               dsum2=0.0
               
               do k=1,nsubl
                  iphip= iphi + nint( ((k-1)*deltaphi*dgr)/ phiinc )
                  
                  if(iphip > nphi)then
 1                   iphip = iphip - nphi
                     if(iphip > nphi) go to 1
                  end if
                  
                  if(iphip < 1)then
 2                   iphip=iphip + nphi
                     if(iphip < 1) go to 2
                  end if
                  
                  z=(k-1)*deltaz + (nzstep-1)*deltadeltaz + zbase
                  iz=z
                  diz=z-iz
                  
                  if(iz > nz-1)then
                     print *,'iz=',iz,' OUT OF BOUNDS - WILL STOP!'
                     stop
                  end if
                  
!***  have iphip, ir, z: interpolate in z
                  
                  den1=dencyl(ir,iphip,iz)
                  den2=dencyl(ir,iphip,iz+1)
                  density=(1.-diz)*den1 + diz*den2
                  
                  dsum=dsum + density
               enddo
               
!***  now have mean density for this ir, iphi, iz      
               
               symcyl( ir, iphi, nzstep) = dsum/nsubl
               
            enddo
         enddo
      enddo

!***  
!***  now generate symmetrized filament in cartesian coordinates
!***  This filament (imageout) will be non-isometric:
!***  dencyl radial scale was same as original Cartesian sampling
!***  new axial scale/section = deltaz (A)/nzlim
      
      ny=nx

      radlim1=irmin*irmin
      radlim2=irmax*irmax

      xcen=nx/2 + 1
      ycen=ny/2 + 1


!**   generate nsubnew subunits to fill nz sections
      nsubnew=(nz/deltaz) + 1
   
      do krep=1, nsubnew
         do kz=1,nzlim
            k=(krep-1)*nzlim + kz
            
            do j=1, ny
               y= j - ycen
               y2=y*y
               
               do i=1, nx
                  x= i-xcen
                  x2=x*x
                  
                  r2=x2+y2
                  
                  if(r2 > radlim2) then
                     imageout(i,j,k)=0.0
                     cycle      ! fortran 90 - end iteration early
                  end if
                  
                  if(r2 < radlim1) then
                     imageout(i,j,k)=0.0
                     cycle      ! fortran 90 - end iteration early
                  end if
                  
!**   interpolate in r, phi (exact in z)
                  
                  
                  if(x == 0.0 .and. y == 0.0) then
                     iphip=1
                  else
                     iphip=nint(dgr*(57.29578*atan2(y,x)-
     $                    (krep-1)*deltaphi)/phiinc)
                  end if
                  
                  if(iphip > nphi) then
 11                  iphip=iphip - nphi
                     if(iphip > nphi)go to 11
                  end if
                  
                  if(iphip < 1) then
 12                  iphip=iphip + nphi
                     if(iphip < 1) go to 12
                  end if
                  
                  r=sqrt(r2)
                  
                  ir=r
                  dir=r-ir
                  
!!!!  may want to use bilinear interpolation here, but more difficult due to "wrap-around" in phi
                  
                  den1 = symcyl(ir,iphip,kz)
                  den2 = symcyl(ir + 1,iphip,kz)
                  
                  density=(1.-dir)*den1 + dir*den2
                  
                  if(k > n_aniso) then
                     print *,'error:', k,n_aniso
!     stop
                  else
                     imageout(i, j, k)= density      
                  end if
                  
               enddo
            enddo
         enddo
      enddo


!******* now have non-isometric density: x,y = scale (A/pixel); z = deltaz (A)/nzlim 
!     *  resample in z so it is isometric

!***   deltaz was changed from A to pixels, to change back = scale*deltaz
!****    zratio=scale/(scale*deltaz/nzlim) = nzlim/deltaz

      zratio=real(nzlim/deltaz)
      
      do  k=1,nz
         zwant=k*zratio
         
         iz=zwant
         zdif=zwant-iz
         
         if(iz > n_aniso)then
            print *,'iz=',iz
            cycle               ! fortran 90 - end iteration early
         end if
         
         do j=1,ny
            do i=1,nx
               den1=imageout(i,j,iz)
               den2=imageout(i,j,iz+1)
               resampout(i,j,k)=(1.-zdif)*den1 + zdif*den2
            enddo
         enddo
      enddo

      end subroutine symmetry


!************************************
      subroutine spider_write(nxout, nyout, nsec, image, 
     $     header, filename)
      implicit none

      integer nheadrec, n, index, ii, i, j, k
      integer nxout, nyout, nsec
      real fmin, fmax, fmean, fsum, fs2
      real header(nxout), header_full(256)
      real image(nxout,nyout,nsec)
      real sigma, ss
      character*60 filename
      character*83 string
      character*12 sdate
      character*8 stime
      equivalence (header_full(212), sdate)
      equivalence (header_full(215), stime)
      logical file_exist
      
      open(1, file=filename, form='unformatted', access='direct',
     $     recl=nxout*4, status='unknown')
      
!***  number of header records
      if(mod(256,nxout) == 0)then
         nheadrec=256/nxout
      else
         nheadrec=(256/nxout) + 1
      end if
      
!     rss      call date(sdate)
!     rss      call time(stime)
      
!***  determine statistics
      fsum=0.0
      fmin=10000.
      fmax=-10000.
      fs2=0.0
      
      do k = 1, nsec
         do j = 1, nyout
            do i = 1, nxout
               fmin=min(fmin, image(i,j,k))
               fmax=max(fmax, image(i,j,k))
               fsum=fsum + image(i,j,k)
               fs2=fs2 + image(i,j,k)**2
            enddo
         enddo
      enddo
      
      n=nxout*nyout*nsec
      fmean=fsum/n
      ss=fs2 - (fsum*fsum/n)
      sigma=sqrt( ss/n )
      
      print *, ' statistics for new SPIDER file:'
      print *, ' fmin=',fmin
      print *, ' fmax=', fmax
      print *, ' fmean=', fmean
      print *, ' sigma=', sigma
      
      do i=1,211
         header_full(i)=0.0
      enddo
      
      do i=217,256
         header_full(i)=0.0
      enddo
      
ccc      do i=212,216
ccc         header_full(i) = 0.0
ccc      enddo

      header_full(1)=nsec
      header_full(2)=nyout
!***  set word 5 to 3 for 3D volume!
      header_full(5)=3
      header_full(6)=1
      header_full(7)=fmax
      header_full(8)=fmin
      header_full(9)=fmean
      header_full(10)=sigma
      header_full(12)=nxout
      header_full(13)=nheadrec
      header_full(22)=nheadrec*nxout*4
      header_full(23)=nxout*4
      
      do i=1,nheadrec
         do j=1,nxout
            index=(i-1)*nxout + j
            if(index <= 256) then
               header(j)=header_full(index)
            else
               header(j)=0.0
            end if 
         enddo
         write(1, REC=i) header
      enddo
      
      ii = nheadrec
      do k= 1,nsec 
         do j= 1,nyout
            ii = ii + 1
            write(1, REC=ii) (image(i,j,k), i=1,nxout)
         enddo
      enddo
      
      close(1)
      
      end subroutine spider_write
